#!/usr/bin/perl
use lib '/home/erealms/ethereal/mgmt/perl';

################################################################################
# Created       : Martin Foster
# Modified      : 01-Jan-2007
################################################################################
#
# Weblog - Weblog functionality built into Ethereal Realms as a news outlet
# Copyright (C) 2000-2007  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 4-3526 Wolfe Cres
#                 Halifax, Nova Scotia
#                 B3L 3S2
#
#################################################################################

use CGI qw(-no_debug -nosticky);				# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use strict;							# Strict variable enforcement

use Ethereal::Database;						# Database handler
use Ethereal::Menu;						# Menu handler
use Ethereal::Param;						# Parameter control
use Ethereal::Template;						# Template handler

#################################################################################
# Gobal override
#################################################################################
$CGI::POST_MAX=1024 * 50;					# Maximum posts
$CGI::DISABLE_UPLOADS = 1;					# Disable uploads

################################################################################
# Data Members
################################################################################
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $menu;							# Menu handle
my $param;							# Parameter handle
my $script;							# Script name

my %board;							# Board hash


################################################################################
# Program Area
################################################################################

	# Initial handles
	$cgi      = new CGI;
	$database = new Ethereal::Database();

	# Connect and fetch
	$database->Connect($cgi);

	# Set title
	$database->{'TITLE'} = $database->{'SYS'}{'TitWeblog'};


	# Parameter handling
	$param = new Ethereal::Param($database, $cgi);
	$param->GetParam();

	# Capture scriptname
        $script = $cgi->url(-relative=>1);


	# Round about way
	if ((length($script) < 2)
	 || ($script =~ /\.(shtml|html)$/i))
	{
		# Assume that this is not the real script
		# Get name and only end
		$script = $0;
		$script =~ s/.*\/(.*)$/$1/gs;
	}


	# Retreive hash
	$database->GetHashBoard(\%board);


	# Display handler
	# Abreviated
	if ($script =~ /^quicklog/i) 
	{
		# Brain dead spacing
		print $cgi->header();

		# Quick view
		QuickView($database, $cgi, $param, \%board);
	}

	# Complete
	else
	{
		#
		# Create instance
		$menu = new Ethereal::Menu($database, $param);

		# Document header
		print $cgi->header();
		print $menu->Rewrite($database->DocumentGetHeader()), "\n";

		# Full view
		PostView($database, $cgi, $param, \%board);

		# Document footer
		print $database->DocumentGetFooter(), "\n";	
	}

#################################################################################
# Sub-Routines
#################################################################################

#####################
# Post Viewer 
#
# Dirt simple functionality that exists simply to display posts in the
# weblog.

sub PostView 
{
	#####################
	# Data members
	my $database = shift;				# Database handle
	my $cgi      = shift;			 	# CGI handle	
	my $param    = shift;				# Parameter list
	my $board    = shift;				# Board hash

	my $res;					# Query results
	my $statement;					# Query handle
	my $tmpl;					# Template handler

	my $count;					# Running counter
	my $entries;					# Remaining entries
	my $offset;					# Offsetting entry
	my $total;					# Total entries
	
	my $inline;					# Inline parameters

	my $pages;					# Pages to display
	my $listing;					# Main listing of entries
	
	my $and;					# Necessary workaround
	my $nam;					# Puppet name
	my $top;					# Topic
        my $dat;					# Date
       	my $tit;					# Title
	my $fil;					# Topic filter
	my $wid;					# Work filter
       
	my %bio;					# Bio links
        my %fil;					# Topic links


	#####################
	# Program area

	# Instance assignment
	$tmpl  = new Ethereal::Template($board);


	# Workaround
	$and = (defined($param->{'FIL'})) ? "AND" : "WHERE";
	
	# Definition
	$fil = (defined($param->{'FIL'})) ? "WHERE LogTopic='" . lc($param->{'FIL'}) . "'" : "";
	$wid = (defined($param->{'WID'})) ? "$and LogIDNumber=$param->{WID}::INT" : "";

	# Safety
	$param->{'OFFSET'} = (defined($param->{'OFFSET'})) ? $param->{'OFFSET'} : 1;
	
	# Cleanup
	$param->Cleanup('WID');


	# Pull amount of entries
	($total) = $database->DataGet("SELECT COUNT(*)
		FROM Weblog $fil $wid");

	# Create offset entries
	$count   = 0;
	$entries = $total;

	# Loop
	do
	{
		# Remove mount to verify spacing
		$entries -= $board->{'SetLimit'};

		# Increment
		$count++;

		# Parameter handling
		$inline = $param->Crypt($param->Flat(), OFFSET=>$count);

		# Create page entry
		$pages = (defined($pages))
			? $pages . ' ' . $tmpl->Pass('TagPage', MLINK=>$database->{'SYS'}{'LnkIntNews'} . $inline, MCOUNT=>$count)
			: $tmpl->Pass('TagPage', MLINK=>$database->{'SYS'}{'LnkIntNews'} . $inline, MCOUNT=>$count);

	} while ($entries > 0);


	# Addition of top
	# Parameter handling
	$inline = $param->Crypt(OFFSET=>1);

	# Append
	$pages .= ' ' . $tmpl->Pass('TagPage', MLINK=>$database->{'SYS'}{'LnkIntNews'} . $inline, MCOUNT=>$board->{'TxtTop'});
	

	# Create offset
	$offset = ($param->{'OFFSET'} - 1) * $board->{'SetLimit'};

		
	# Find and list
	# Prepare and execute	
	$database->Pull(\$statement, "SELECT
		  LogIDNumber AS \"LogIDNumber\",
		  PuppetName  AS \"PuppetName\",
		  LogTitle    AS \"LogTitle\",
		  LogTopic    AS \"LogTopic\",
		  LogPost     AS \"LogPost\",
		  getDate(LogTimestamp) AS \"LogTimestamp\"
		FROM Weblog $fil $wid
		ORDER BY LogIDNumber DESC
		LIMIT  $board->{SetLimit}
		OFFSET $offset");

	# Safety
	$count   = 0;
	$listing = '';

	
	# Loop through results
	while ($res = $statement->fetchrow_hashref())
	{
		# Increment
		$count++;

		# Assign
		$nam = $res->{'PuppetName'};
		$top = $res->{'LogTopic'};
		$dat = $res->{'LogTimestamp'};


		# Define hashes
		# Biographical data
		$bio{$nam} = (defined($bio{$nam}))
		  ? $bio{$nam}
		  : '/' . join('', $database->DataGetPuppetShortname($nam)) . '/system';

		# Topic links
		$fil{$top} = (defined($fil{$top}))
		  ? $fil{$top} 
		  : $param->Crypt(OFFSET=>1, FIL=>$top);


		# Append spacer
		$listing .= $board->{'TmplSpacer'} if ($count > 1);
		
		# Append to listing
		$listing .= $tmpl->Pass('TmplWEntry',
			LWBIO => $database->{'SYS'}{'LnkIntBio'}  . $bio{$nam},
			LWTOP => $database->{'SYS'}{'LnkIntNews'} . $fil{$top},
			MWDAT => $dat,
			MWPUP => $nam,
			MWTOP => uc($top),
			MWBDY => $res->{'LogPost'},
			MWTIT => $res->{'LogTitle'}
		 );
	}

	# Finish query
	$statement->finish();


	# Display entries
	$tmpl->Show('TmplWeblog',
		LSTENTRY => $listing,
		MCOUNT   => $count,
		MTOTAL   => $total,
		MPAGES   => $pages
	 );
}


#####################
# Quick Viewer 
#
# Used in conjunction with SSI as to be able to display a part of the log
# without including the rest of the interface.  Very useful for front
# pages.

sub QuickView 
{
	#####################
	# Data members
	my $database = shift;				# Database handle
	my $cgi      = shift;			 	# CGI handle	
	my $param    = shift;				# Parameter list
	my $board    = shift;				# Board hash

	my $tmpl;					# Template handler
	
	my $res;					# Query results
	my $statement;					# Query handle

	my $dat;					# Date/time stamp
	my $dis;					# Display amount widget
	my $fil;					# Filtering by topic
	my $nam;					# Discovered name
	my $num;					# Number to display
	my $top;					# Currently selected topic

	my $inl;					# Inline information
	my $sub;					# Sub information

	my %fil;					# Filter inline hash
	my %bio;					# Bio inline hash


	#####################
	# Program area

	# Create instance
	$tmpl  = new Ethereal::Template($board);

	# Find and list
	# Prepare and execute	
	$database->Pull(\$statement, "SELECT
		  LogIDNumber AS \"LogIDNumber\",
		  PuppetName  AS \"PuppetName\",
		  LogTitle    AS \"LogTitle\",
		  LogTopic    AS \"LogTopic\",
		  LogPost     AS \"LogPost\",
		  GetDate(LogTimestamp) AS \"LogTimestamp\"
		FROM Weblog
		ORDER BY LogIDNumber DESC
		LIMIT 3");

	# Loop and display
	while ($res = $statement->fetchrow_hashref())
	{
		# Value handling
		# Assign
		$nam =  $res->{'PuppetName'};
		$top =  $res->{'LogTopic'};

		# Pull login name
		($dat) = $res->{'LogTimestamp'};

		# Define hashes
		$bio{$nam} = (defined($bio{$nam})) ? $bio{$nam} : $param->Crypt(BCHAR=>$nam, BPOOL=>'system');
		$fil{$top} = (defined($fil{$top})) ? $fil{$top} : $param->Crypt(FIL=>$top);

		# Inline parameter
		$inl = $param->Crypt(WID=>$res->{'LogIDNumber'});


		# Create display
		$tmpl->Show('TmplQuickNews',
			MDATET => $dat,
			MPUPPT => $nam,
			MTITLE => $res->{'LogTitle'},
			MTOPIC => uc($top),
			LTOPIC => $database->{'SYS'}{'LnkIntNews'} . $fil{$top},
			LENTRY => $database->{'SYS'}{'LnkIntNews'} . $inl,
			LBIOGR => $database->{'SYS'}{'LnkIntBio'}   . $bio{$nam}
		 );
	}

	# Finish query
	$statement->finish();
}
